import { localize } from 'vscode/vscode/vs/nls';
import { deepClone } from 'vscode/vscode/vs/base/common/objects';
import { ProblemMatcherRegistry } from './problemMatcher.js';
import schema$1 from './jsonSchemaCommon.js';

const schema = {
    oneOf: [
        {
            allOf: [
                {
                    type: 'object',
                    required: ['version'],
                    properties: {
                        version: {
                            type: 'string',
                            enum: ['0.1.0'],
                            deprecationMessage: ( localize(6533, 'Task version 0.1.0 is deprecated. Please use 2.0.0')),
                            description: ( localize(6534, 'The config\'s version number'))
                        },
                        _runner: {
                            deprecationMessage: ( localize(6535, 'The runner has graduated. Use the official runner property'))
                        },
                        runner: {
                            type: 'string',
                            enum: ['process', 'terminal'],
                            default: 'process',
                            description: ( localize(
                                6536,
                                'Defines whether the task is executed as a process and the output is shown in the output window or inside the terminal.'
                            ))
                        },
                        windows: {
                            $ref: '#/definitions/taskRunnerConfiguration',
                            description: ( localize(6537, 'Windows specific command configuration'))
                        },
                        osx: {
                            $ref: '#/definitions/taskRunnerConfiguration',
                            description: ( localize(6538, 'Mac specific command configuration'))
                        },
                        linux: {
                            $ref: '#/definitions/taskRunnerConfiguration',
                            description: ( localize(6539, 'Linux specific command configuration'))
                        }
                    }
                },
                {
                    $ref: '#/definitions/taskRunnerConfiguration'
                }
            ]
        }
    ]
};
const shellCommand = {
    type: 'boolean',
    default: true,
    description: ( localize(
        6540,
        'Specifies whether the command is a shell command or an external program. Defaults to false if omitted.'
    ))
};
schema.definitions = deepClone(schema$1.definitions);
const definitions = schema.definitions;
definitions['commandConfiguration']['properties']['isShellCommand'] = deepClone(shellCommand);
definitions['taskDescription']['properties']['isShellCommand'] = deepClone(shellCommand);
definitions['taskRunnerConfiguration']['properties']['isShellCommand'] = deepClone(shellCommand);
Object.getOwnPropertyNames(definitions).forEach(key => {
    const newKey = key + '1';
    definitions[newKey] = definitions[key];
    delete definitions[key];
});
function fixReferences(literal) {
    if (Array.isArray(literal)) {
        literal.forEach(fixReferences);
    }
    else if (typeof literal === 'object') {
        if (literal['$ref']) {
            literal['$ref'] = literal['$ref'] + '1';
        }
        Object.getOwnPropertyNames(literal).forEach(property => {
            const value = literal[property];
            if (Array.isArray(value) || typeof value === 'object') {
                fixReferences(value);
            }
        });
    }
}
fixReferences(schema);
ProblemMatcherRegistry.onReady().then(() => {
    try {
        const matcherIds = ( (( (ProblemMatcherRegistry.keys())).map(key => '$' + key)));
        definitions.problemMatcherType1.oneOf[0].enum = matcherIds;
        definitions.problemMatcherType1.oneOf[2].items.anyOf[1].enum = matcherIds;
    }
    catch (err) {
        console.log('Installing problem matcher ids failed');
    }
});

export { schema as default };
